library cool_stepper;

export 'package:cool_stepper_custom/src/models/cool_step_custom.dart';
export 'package:cool_stepper_custom/src/models/cool_stepper_config_custom.dart';

import 'package:another_flushbar/flushbar.dart';

import 'package:cool_stepper_custom/src/models/cool_step_custom.dart';
import 'package:cool_stepper_custom/src/models/cool_stepper_config_custom.dart';
import 'package:cool_stepper_custom/src/widgets/cool_stepper_view_custom.dart';
import 'package:flutter/material.dart';

/// CoolStepperCustom
class CoolStepperCustom extends StatefulWidget {
  /// The steps of the stepper whose titles, subtitles, content always get shown.
  ///
  /// The length of [steps] must not change.
  final List<CoolStepCustom> steps;

  /// Actions to take when the final stepper is passed
  final VoidCallback onCompleted;

  /// Padding for the content inside the stepper
  final EdgeInsetsGeometry contentPadding;

  /// CoolStepperCustom config
  final CoolStepperCustomConfig config;

  /// This determines if or not a snackbar displays your error message if validation fails
  ///
  /// default is false
  final bool showErrorSnackbar;

  const CoolStepperCustom({
    Key? key,
    required this.steps,
    required this.onCompleted,
    this.contentPadding = const EdgeInsets.symmetric(horizontal: 20.0),
    this.config = const CoolStepperCustomConfig(),
    this.showErrorSnackbar = false,
  }) : super(key: key);

  @override
  _CoolStepperCustomState createState() => _CoolStepperCustomState();
}

class _CoolStepperCustomState extends State<CoolStepperCustom> {
  PageController? _controller = PageController();

  int currentStep = 0;

  @override
  void dispose() {
    _controller!.dispose();
    _controller = null;
    super.dispose();
  }

  Future<void>? switchToPage(int page) {
    _controller!.animateToPage(
      page,
      duration: const Duration(milliseconds: 300),
      curve: Curves.ease,
    );
  }

  bool _isFirst(int index) {
    return index == 0;
  }

  bool _isLast(int index) {
    return widget.steps.length - 1 == index;
  }

  void onStepNext() {
    final validation = widget.steps[currentStep].validation!();

    /// [validation] is null, no validation rule
    if (validation == null) {
      if (!_isLast(currentStep)) {
        setState(() {
          currentStep++;
        });
        FocusScope.of(context).unfocus();
        switchToPage(currentStep);
      } else {
        widget.onCompleted();
      }
    } else {
      /// [showErrorSnackbar] is true, Show error snackbar rule
      if (widget.showErrorSnackbar) {
        final flush = Flushbar(
          message: validation,
          flushbarStyle: FlushbarStyle.FLOATING,
          margin: EdgeInsets.all(8.0),
          borderRadius: BorderRadius.all(Radius.circular(8.0)),
          icon: Icon(
            Icons.info_outline,
            size: 28.0,
            color: Theme.of(context).primaryColor,
          ),
          duration: Duration(seconds: 2),
          leftBarIndicatorColor: Theme.of(context).primaryColor,
        );
        flush.show(context);

        // final snackBar = SnackBar(content: Text(validation));
        // ScaffoldMessenger.of(context).showSnackBar(snackBar);
      }
    }
  }

  void onStepBack() {
    if (!_isFirst(currentStep)) {
      setState(() {
        currentStep--;
      });
      switchToPage(currentStep);
    }
  }

  @override
  Widget build(BuildContext context) {
    final content = Expanded(
      child: PageView(
        controller: _controller,
        physics: NeverScrollableScrollPhysics(),
        children: widget.steps.map((step) {
          return CoolStepperCustomView(
            step: step,
            contentPadding: widget.contentPadding,
            config: widget.config,
          );
        }).toList(),
      ),
    );

    final counter = Container(
      child: Text(
        "${widget.config.stepText ?? 'STEP'} ${currentStep + 1} ${widget.config.ofText ?? 'OF'} ${widget.steps.length}",
        style: TextStyle(
          fontWeight: FontWeight.bold,
        ),
      ),
    );

    String getNextLabel() {
      String nextLabel;
      if (_isLast(currentStep)) {
        nextLabel = widget.config.finalText ?? 'FINISH';
      } else {
        if (widget.config.nextTextList != null) {
          nextLabel = widget.config.nextTextList![currentStep];
        } else {
          nextLabel = widget.config.nextText ?? 'NEXT';
        }
      }
      return nextLabel;
    }

    String getPrevLabel() {
      String backLabel;
      if (_isFirst(currentStep)) {
        backLabel = '';
      } else {
        if (widget.config.backTextList != null) {
          backLabel = widget.config.backTextList![currentStep - 1];
        } else {
          backLabel = widget.config.backText ?? 'PREV';
        }
      }
      return backLabel;
    }

    final buttons = Container(
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[
          Padding(
            padding: widget.config.backButtonPadding ?? EdgeInsets.all(0.0),
            child: TextButton(
              onPressed: onStepBack,
              style: widget.config.backButtonStyle,
              child: Text(
                getPrevLabel(),
                style: widget.config.backTextStyle ??
                    TextStyle(color: Colors.grey),
              ),
            ),
          ),
          counter,
          Padding(
            padding: (_isLast(currentStep)
                    ? widget.config.finalButtonPadding
                    : widget.config.nextButtonPadding) ??
                EdgeInsets.all(0.0),
            child: TextButton(
              onPressed: onStepNext,
              style: _isLast(currentStep)
                  ? widget.config.finalButtonStyle
                  : widget.config.nextButtonStyle,
              child: Text(
                getNextLabel(),
                style: (_isLast(currentStep)
                        ? widget.config.finalTextStyle
                        : widget.config.nextTextStyle) ??
                    TextStyle(
                      color: Colors.green,
                    ),
              ),
            ),
          ),
        ],
      ),
    );

    return Container(
      child: Column(
        children: [content, buttons],
      ),
    );
  }
}
